﻿/* TITLE: QuikESignTransport
 * DESCRIPTION: Quik! E-signature Transport Web Service for use with Docusign
 * AUTHOR: Efficient Technology, Inc
 * COPYRIGHT: This project is written by Efficient Technology, Inc. 2013 and distributed as part of the Quik! Forms Engine software license.
 *            Customers may modify this project as necessary to meet their specific needs. This project is not intended for distribution
 *            or resale but is intended for use in a customer's production system that uses the Quik! Forms Engine software.
 *
 * 
 * This SELF-SERVICE project, including source code, is available to Quik! customers to enable signing Quik! forms with Docusign.
 * 
 * INSTRUCTIONS:
 * Deploy this project on your web server to manage e-sign transactions initiated by users clicking a SIGN button on
 * a Quik! form. When a user clicks the SIGN button this web service will be called so that the form data can be
 * sent to the Quik! server. The Quik! server will then create and return the Docusign XML data and final PDF needed
 * for a Docusign envelope. This web service will then send that data to Docusign to create and initiate an envelope.
 * 
 * DO NOT CHANGE
 * Do not change the following web.config settings or this project won't connect to Quik! properly
 *     <applicationSettings>
 *       <QuikESignTransportWS.Properties.Settings>
 *           <setting name="QuikESignTransportWS_DocuSignData_GetDocusignData"
 *               serializeAs="String">
 *               <value>https://websvcs.quikforms.com/Docusign/EsignWS/GetDocusignData.asmx</value>
 *           </setting>
 *       </QuikESignTransportWS.Properties.Settings>
 *   </applicationSettings>
 *   
 *      
 * DO CHANGE
 * The following keys need to be set by you in order to use this web service
 *  <add key="strAPIAccountId" value="YOUR DOCUSIGN API ACCOUNT ID"/>
 *  <add key="strAPIUserName" value="YOUR DOCUSIGN API USERNAME"/>
 *  <add key="strUserPassword" value="YOUR DOCUSIGN API USER PASSWORD"/>
 *  <add key="strAPIUrl" value="https://demo.docusign.net/api/3.0/api.asmx"/> <!--Change this to the production API (https://www.docusign.net/api/3.0/credential.asmx) once certified by Docusign IF you modify this project. Otherwise, use the production URL now.-->
 *  <add key="SupportEmail" value ="YOUR SUPPORT EMAIL ADDRESS"/>
 *  <add key="SupportPhone" value ="YOUR SUPPORT PHONE NUMBER"/>
 *
 * ALSO change the following keys in the <client> section to production URLs for Docusign
 *  <client>
 *    <endpoint address="https://demo.docusign.net/api/3.0/credential.asmx" binding="basicHttpBinding" bindingConfiguration="CredentialSoap" contract="CredentialService.CredentialSoap" name="CredentialSoap"/>
 *    <!--Change this to the production API (https://www.docusign.net/api/3.0/credential.asmx) once certified by Docusign IF you modify this project. Otherwise, use the production URL now.-->
 *    <endpoint address="https://demo.docusign.net/api/3.0/api.asmx" binding="basicHttpBinding" bindingConfiguration="APIServiceSoap" contract="APIService.APIServiceSoap" name="APIServiceSoap"/>
 *    <!--Change this to the production API (https://www.docusign.net/api/3.0/api.asmx) once certified by Docusign IF you modify this project. Otherwise, use the production URL now.-->
 *  </client>
 * 
 * CERTIFICATION
 * This project, as-is, is certified to work with Docusign and Quik!. Upon modifying this project you may need to re-certify
 * for your own integration.
 * 
 * STEPS
 * 1. When running the Quik! Forms Engine (QuikFormsEngine.DLL), populate the SignURL property with the URL to this web service
 *   (e.g. objQFE.SignURL = "https://YOUR-DOMAIN.com/QuikSignTransport.aspx")
 *   
 * 2. When the user clicks the SIGN button on a Quik! Form the form data will be posted to this page.
 *  a. This page will capture the form data and pass it to a Quik! server for processing.
 *  b. The Quik! server will prepare a PDF, add in any external documents to the package and create the Docusign XML file.
 *  c. This web service will receive back a PDF and XML file that is ready to be sent to Docusign.
 *  d. By default this web service will then send the PDF and XML to Docusign to create and initiate an envelope, returning a Docusign
 *     EnvelopeID. Optionally, you can stop the process and submit the envelope to Docusign yourself at a later time.
 * 
 * 3. Write your own API calls to Docusign to poll for the envelope's status and to download final documents using the Envelope ID
 * 
 * DOCUSIGN API PROJECT
 * This project relies upon a compiled DLL called EnvelopeCreatorProject, written by Efficient Technology Inc. The source code for this
 * project is also available to assist with your Docusign API integration. The source code can be found in the Quik! Forms Engine
 * software contents file (QuikFormsEngine.zip archive).
 * 
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;
using System.Collections;
using System.Text;
using System.Xml;
using Newtonsoft.Json;
using System.Net;
using EnvelopeCreatorProject;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using log4net;
using Microsoft.VisualBasic;
using System.Data;

namespace QuikESignTransportWS
{
    public partial class QuikSignTransport : System.Web.UI.Page
    {

        private ILog logger = log4net.LogManager.GetLogger("QuikSignTransport");

        private String createJSON(String statusCode, String statusMessage)
        {
            logger.Debug("JSON: " + "{\"StatusCode\":\"" + statusCode + "\",\"StatusMessage\":\"" + statusMessage + "\"}");
            return JsonConvert.SerializeObject(new { StatusCode = statusCode, StatusMessage = statusMessage }).ToString();
        }

        private String FileName()
        {
            String strFileName = DateTime.Now.Ticks.ToString() + ".pdf";
            logger.Debug("fileName: " + strFileName);
            return strFileName;
        }

        private String DocuSignTransporter(string endpointAddress, DataTable dtFiles, XmlDocument XMLDataEntryPath, string unid)
        {
            EnvelopeCreatorProject.EnvelopeTransporterCreator objEnvelopeCreator = new EnvelopeCreatorProject.EnvelopeTransporterCreator();
            String PDFName = FileName();
            String strResult = objEnvelopeCreator.CreateEnvelopeTransporter(endpointAddress, dtFiles, unid, XMLDataEntryPath);
            return strResult;
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (Request.ContentLength.Equals(0))
            {
                logger.Debug("Request content is empty.");
                return;
            }
            RemoteCertificateValidationCallback callback = delegate(object sender2, X509Certificate cert, X509Chain chain, SslPolicyErrors sslError)
            {
                logger.Debug("RemoteCertificateValidationCallback.");
                return true;
            };
            ServicePointManager.ServerCertificateValidationCallback = new RemoteCertificateValidationCallback(callback);

            StreamReader sr = new StreamReader(Request.InputStream);
            string strFormData = sr.ReadToEnd();
            logger.Debug("Form Data received from sign event: " + strFormData);
            string strDecodedFormData = strFormData;
            try
            {

                string[] StringArray = strDecodedFormData.Split(new string[] { "&BeginSign&" }, System.StringSplitOptions.None);
                string strFormFieldsData = StringArray[0];
                string strFormSignData = StringArray[1];


                string[] StringDataSign = strFormSignData.Split(new string[] { "&EndSign&" }, System.StringSplitOptions.None);
                string strRecipientData = StringDataSign[0];
                string strSignFieldData = StringDataSign[1];


                //Call the Quik! server to get the Docusign data (XML)
                DocuSignData.GetDocusignData wsGetDocusignData = new DocuSignData.GetDocusignData();
                DocuSignData.QuikResultWS result = wsGetDocusignData.GetDocusignResources(strFormFieldsData, strRecipientData, strSignFieldData);

                String json = this.createJSON(result.ErrorCode.ToString(), result.Message);

                if (result.ErrorCode.ToString().Equals("0"))
                {
                    DataTable dtFiles = result.ResultData.Tables["PDFFiles"];
                    DataTable dtXMLData = result.ResultData.Tables["DSignXML"];
                    string XML = dtXMLData.Rows[0]["DSignXML"].ToString();
                    XmlDocument XmlDoc = new XmlDocument();
                    XmlDoc.LoadXml(Server.UrlDecode(XML));


                    //Get UNID
                    string unid = ReturnUNID(strFormFieldsData);

                    //Get the endpoint address for Docusign, based on the environment ID
                    string endpointAddress = dtXMLData.Rows[0]["DSignEnvironmentURL"].ToString();;

                    //Call Docusign to send, create and initiate the envelope
                    string strEnvelopeID = this.DocuSignTransporter(endpointAddress,dtFiles, XmlDoc, Encryption.Decrypt(unid));
                    logger.Debug("Envelope ID: " + strEnvelopeID);
                    /* NOTE:
                     * If you want to stop the process here and take the document and XML to do with as you wish, then dim out the above
                     * line and call the Docusign API on your own at a later stage. Otherwise, the above line will create an envelope right now
                     * and send the documents out for signature, per the envelope instructions and content.
                     * 
                     * Use strEnvelopID to call the Docusign API for additional needs (e.g. polling for status, downloading final documents, etc.)
                     */
                }
                //Return response to the form
                Response.ContentType = "application/json";
                logger.Debug("JSON Response: " + json);
                Response.Write(json);

            }
            catch (Exception ex)
            {
                logger.Error("Response from QuikSignTransportFailed. Reason: " + ex.Message + ". " + ex.StackTrace.ToString());

                string strSupportEmail = System.Configuration.ConfigurationManager.AppSettings["SupportEmail"];
                string strSupportPhone = System.Configuration.ConfigurationManager.AppSettings["SupportPhone"];
                string strErrorCode = "1111";

                //Customize your error response here
                String json = this.createJSON(strErrorCode, "MESSAGE:\nPlease print your document and contact the administrator at " + strSupportEmail + " or " + strSupportPhone + ". \n\nDETAILS:\n" + ex.Message);
                Response.Write(json);
            }

            Response.AddHeader("Access-Control-Allow-Origin", "*");
            Response.AddHeader("Access-Control-Allow-Headers", "Content-Type");
            Response.End();

        }


        private string ReturnUNID(string strFormData)
        {
            //Parse the form data
            string[] strData = null;
            strData = strFormData.Split(new string[]{"&"}, StringSplitOptions.None);

            //Find the form request UNID
            string strUNID = "";
            int intCount = 0;

            for (intCount = Information.LBound(strData); intCount <= Information.UBound(strData); intCount++)
            {
                if (Strings.InStr(strData[intCount], "QFVUNID") > 0)
                {
                    strUNID = HttpUtility.UrlDecode(Strings.Replace(HttpUtility.UrlDecode(strData[intCount]), "QFVUNID=UNID=", "", 1, 1));
                    //URLDecode twice to account for the equal sign "UNID=" which was encoded before the UNID was encoded.
                    return strUNID;
                }
            }

            return strUNID;

        }


        private int GetEnvironmentIDFromFormData(string HTMLData)
        {
            int environmentID = 0;
            try
            {
                //Parse the form data
                string[] strData = null;
                strData = HTMLData.Split(new string[]{"&"},StringSplitOptions.None);

                //Find the form request UNID
                int intCount = 0;
                for (intCount = Information.LBound(strData); intCount <= Information.UBound(strData); intCount++)
                {
                    if (Strings.InStr(strData[intCount], "SignEnvironmentID") >= 0)
                    {
                        environmentID = Convert.ToInt32(Strings.Replace(strData[intCount], "SignEnvironmentID=", ""));
                        break; 
                    }
                }
            }
            catch (Exception ex)
            {
                environmentID = 1;
                //PROD -> Default value
            }
            return environmentID;
        }

    }


}